// ServerDlg.h : header file
//

#if !defined(AFX_SERVERDLG_H__62B8F5F2_D70D_11D6_8B1E_00065B874763__INCLUDED_)
#define AFX_SERVERDLG_H__62B8F5F2_D70D_11D6_8B1E_00065B874763__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

/////////////////////////////////////////////////////////////////////////////
// CServerDlg dialog

#include "..\SocketInterface.h"

/*
This acts as a telnet chat-server, allowing you to connect to this server using a standard Telnet client.
There are the usual C++ problems of where to declare things when
the Dialog uses the Server, the Server uses the Dialog,
the Server uses the Client and the Client uses the Server...
Those problems will ususally sort themselves out if you implement the classes is the cpp file (I kept all their code in the .h file so that you can see everything quickly and easily).

Since Telnet doesn't know what CStrings are (CStrings are what the SocketInterface classes use by default) the Sender and GetReply functions are overridden.

While a Telnet client is typing, any incoming messages queue up in the Message Queue and they all appear as soon as the client hits return.
To see this, open two telnet sessions and connect to the server; type a single letter one one session, but don't press [Return], then on the other session type lots of lines of text. When you press return on the first session, all that you typed on the second session now appears.
This demonstrates corect usage of the Message Queue and that the sessions are handled by different Threads on the Server.

If the client sends a line of text:
Bye
the Server disconnects that client.
*/

class CChat;
class CServerDlg;

class CChatServer : public CSocketServer {
  CServerDlg* Dlg;
public:
  CChatServer(BYTE Port, CServerDlg* Dlg) : CSocketServer(Port), Dlg(Dlg) {}
  void Say(CString Msg);
protected:
  virtual CConnection* New();
};

class CServerDlg : public CDialog {
  CChatServer* Chat;

// Construction
public:
  CServerDlg(CWnd* pParent=NULL) : CDialog(CServerDlg::IDD, pParent), Chat(0) {}	// standard constructor
  virtual ~CServerDlg();
  void Say(CString Msg) {if(::IsWindow(m_hWnd)) ((CListBox*)GetDlgItem(IDC_LIST1))->AddString(Msg);}

// Dialog Data
	//{{AFX_DATA(CServerDlg)
	enum { IDD = IDD_SERVER_DIALOG };
		// NOTE: the ClassWizard will add data members here
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CServerDlg)
	//}}AFX_VIRTUAL

// Implementation
protected:

	// Generated message map functions
	//{{AFX_MSG(CServerDlg)
	virtual BOOL OnInitDialog();
	afx_msg void OnBroadcast();
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

class CChat : public CConnection {
  void Say      (CString Msg) {((CChatServer*)GetServer())->Say(ID+Msg);}
  void Broadcast(CString Msg) {Say(Msg); ((CChatServer*)GetServer())->Broadcast(ID+Msg);}
  CString ID;
  CString GetReply() {
    CString S;
    try {
      ArI->ReadString(S); // It'll wait here for as long as the connection is OK. If the connection is broken it returns an empty string.
    }catch(...) {return ''+sConnectionBroken;}
    return S;
  }
public:
  CChat(CSocketServer* Me) : CConnection(Me) {}
protected:
  void Sender(CString Msg) {ArO->WriteString(Msg+"\r\n"); ArO->Flush();}
  void OnDisconnect() {Broadcast("Disconnected");}
  void OnConnect() {
    ID.Format("%s(%i): ", RemoteIP, m_hSocket);
    Broadcast("Connected");
    Send("You will appear as "+ID);
  }
  void OnReceive() {
    CString S(GetReply());
    if(*S=='') {Say(S.Mid(1)); return;}
    if(!S.CompareNoCase("Bye")) Disconnect();
    Broadcast(S);
  }
};

//{{AFX_INSERT_LOCATION}}
// Microsoft Visual C++ will insert additional declarations immediately before the previous line.

#endif // !defined(AFX_SERVERDLG_H__62B8F5F2_D70D_11D6_8B1E_00065B874763__INCLUDED_)
